import 'package:http/http.dart' as http;
import 'package:html/dom.dart';
import 'package:html/parser.dart';
import 'package:http/http.dart';

class NavScraper {
  String _cookie;
  String rut;
  String pass;

  NavScraper({this.rut, this.pass});

  Future<bool> login() async {
    http.Response loginResponse = await http.post(
      'https://nave13.ucv.cl/index.php',
      body: {
        'rut_num': rut,
        'user_pas': pass
      }
    );
    if (!loginResponse.body.contains('HTTP-EQUIV=Refresh')) return false;
    _cookie = loginResponse.headers['set-cookie'];
    return true;
  }

  Future<String> getName() async {
    if (_cookie == null) return null;
    Document document = await getNavDocument('https://nave13.ucv.cl/alumno/fichas/ficha_info_matricula.php');
    List<Element> nombres = document.querySelectorAll('div.col-md-8 > p');
    return nombres[0].innerHtml.trim();
  }

  Future<List<Map<String, String>>> getCourses() async {
    if (_cookie == null) return null;
    List courses = new List<Map<String, String>>();
    Document document = await getNavDocument('https://nave13.ucv.cl/alumno/fichas/ficha_consolidada.php#');
    List<Element> links = document.querySelectorAll('td > a');
    for (Element link in links) {
      String onClick = link.attributes['onclick'];
      if (onClick.contains('enviar_curso')) {
        List<String> courseData = onClick.substring(onClick.indexOf('(') + 1, onClick.indexOf(')')).split(',');
        courses.add({
          'c_asignatura': courseData[0],
          'n_periodo': courseData[1],
          't_periodo': courseData[2],
          'ano': courseData[3],
          'paralelo': courseData[4]
        });
      }
    }
    return courses;
  }

  Future<List<Map<String, String>>> getCourseData({String c_asignatura, String n_periodo, String t_periodo, String ano, String paralelo}) async {
    List courseData = List<Map<String, String>>();

    Response fichaResponse = await http.post(
      'https://nave13.ucv.cl/docencia/cursos/ficha_curso.php',
      headers: {'Cookie': _cookie},
      body: {
        'c_asignatura': c_asignatura,
        'n_periodo': n_periodo,
        't_periodo': t_periodo,
        'ano': ano,
        'paralelo': paralelo
      }
    );

    if (fichaResponse.body.contains('Falta el par&aacute;metro requerido'))
      return null;

    Document fichaDocument = parse(fichaResponse.body.substring(fichaResponse.body.indexOf('Horario')));
    List<Element> references = fichaDocument.querySelectorAll('div.table-responsive > table');

  
    for (Element catedra in references[0].children[1].children){
      courseData.add({
        'dia' : catedra.children[0].innerHtml,
        'clave' : catedra.children[1].innerHtml.substring(0, catedra.children[1].innerHtml.indexOf('(')),
        'sala' : catedra.children[2].children[0].innerHtml,
        'tipo' : 'catedra'
      });
    }

    if(references.length > 1)
      for (Element catedra in references[1].children[1].children){
        courseData.add({
          'dia' : catedra.children[0].innerHtml,
          'clave' : catedra.children[1].innerHtml.substring(0, catedra.children[1].innerHtml.indexOf('(')),
          'sala' : catedra.children[2].children[0].innerHtml,
          'tipo' : 'ayudantia'
        });
      } 

    return courseData;
  }

  Future<Document> getNavDocument(String link) async {
    return parse((await http.get(link, headers: {'Cookie': _cookie})).body);
  }
}
